#define MODULE
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/mc146818rtc.h>

#include <asm/system.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>

#include <rtai.h>
#include <rtl_fifo.h>
#include <rtl_sched.h>

#define ONESHOT

#define TICK_PERIOD 125000

#define STACK_SIZE 4000

RT_TASK thread;

static int cpu_used[NR_RT_CPUS];
static int filter(int x)
{
	static int oldx;
	int ret;

	if (x & 0x80) {
		x = 382 - x;
	}
	ret = x > oldx;
	oldx = x;
	return ret;
	
}


void intr_handler(int t) {
	char data;
	char temp;
	while(1) {
		cpu_used[hard_cpu_id()]++;
		if (rtf_get(0, &data, 1) > 0) {
			data = filter(data);
			temp = inb(0x61);            
			temp &= 0xfd;
			temp |= (data & 1) << 1;
			outb(temp,0x61);
		}
		rt_task_wait_period();
	}
}


int init_module(void)
{
	RTIME now, tick_period;
	rtf_create(0, 12000);
	rt_task_init(&thread, intr_handler, 0, STACK_SIZE, 0, 0, 0);
#ifdef ONESHOT
	rt_set_oneshot_mode();
#endif
	tick_period = start_rt_timer(nano2count(TICK_PERIOD));
	now = rt_get_time() + tick_period;
	rt_task_make_periodic(&thread, now, tick_period);
	return 0;
}


void cleanup_module(void)
{
	int cpuid;
	stop_rt_timer();
	rtf_destroy(0);
	rt_task_delete(&thread);
	printk("\n\nCPU USE SUMMARY\n");
	for (cpuid = 0; cpuid < NR_RT_CPUS; cpuid++) {
		printk("# %d -> %d\n", cpuid, cpu_used[cpuid]);
	}
	printk("END OF CPU USE SUMMARY\n\n");
}
